import _extends from "@babel/runtime/helpers/esm/extends";
import { getConfig, isClient, setConfig } from './config.js';
import analytics from './segmentWrapper.js';

/**
 * Cookie to extract the already saved consents for the user
 * @type {string}
 */
var TCF_COOKIE_KEY = 'borosTcf';
/**
 * TCF Api Version to use
 * @type {number}
 */
var TCF_API_VERSION = 2;

/**
 * Default properties to send with every TCF tracking event
 */
var TCF_TRACK_PROPERTIES = {
  channel: 'GDPR'
};

/**
 * List of purpose ids needed to be able to track with all info
 * @type {Array<number>}
 */
var NEEDED_PURPOSES = {
  analytics: [1, 8, 10],
  advertising: [3]
};

/**
 * TCF events
 */
var TCF_EVENTS = {
  // Event that determines that the tcData has been loaded
  LOADED: 'tcloaded',
  // Event that determines that the user has performed an action
  USER_ACTION_COMPLETE: 'useractioncomplete'
};

/**
 * State of user according to GDPR regarding tracking
 */
export var USER_GDPR = {
  ACCEPTED: 'accepted',
  DECLINED: 'declined',
  UNKNOWN: 'unknown'
};

/**
 * Define the user GDPR consents state. This value will be updated with new values
 * when the consents of the users changes.
 */
var gdprState = {
  listeners: [],
  value: undefined,
  addListener: function addListener(callback) {
    return gdprState.listeners.push(callback);
  },
  get: function get() {
    return gdprState.value;
  },
  set: function set(value) {
    var _ref = value || {},
      analytics = _ref.analytics,
      advertising = _ref.advertising;
    gdprState.value = {
      analytics: analytics,
      advertising: advertising
    };
    gdprState.listeners.forEach(function (fn) {
      return fn(value);
    });
    gdprState.listeners = [];
  }
};

/**
 * Read cookie by using a key
 * @returns {string}
 */
function readCookie(key) {
  var re = new RegExp(key + '=([^;]+)');
  var value = re.exec(document.cookie);
  return value !== null ? unescape(value[1]) : null;
}

/**
 * Check if we're on client and tcfApi is available on window object
 * @returns {Boolean}
 */
var checkTcfIsAvailable = function checkTcfIsAvailable() {
  // if we're on the client, check we haven't already initialized it
  if (getConfig('initialized')) return false;

  // if we're on client, check if we have the tcfapi available
  var isTcfApiAvailable = !!window.__tcfapi;
  !isTcfApiAvailable && console.warn("[tcfTracking] window.__tcfapi is not available on client and TCF won't be tracked.");
  return isTcfApiAvailable;
};

/**
 * Check from a list of consents if user has accepted being tracked
 * @param {{[purposeId: string]: boolean}} userConsents
 * @returns {Boolean}
 */
var checkHasUserConsentedAnalytics = function checkHasUserConsentedAnalytics(userConsents) {
  return NEEDED_PURPOSES.analytics.every(function (purposeId) {
    return userConsents["" + purposeId];
  });
};
var checkHasUserConsentedAdvertising = function checkHasUserConsentedAdvertising(userConsents) {
  return NEEDED_PURPOSES.advertising.every(function (purposeId) {
    return userConsents["" + purposeId];
  });
};

/**
 * Track a specific TCF event
 * @param {object} params
 * @param {string} params.eventId Event ID to be sent with the TCF Tracking
 * @param {string} params.gdprPrivacy Send a string telling if the gdpr has been accepted or reject
 * @return {Promise}
 */
var trackTcf = function trackTcf(_ref2) {
  var eventId = _ref2.eventId,
    gdprPrivacy = _ref2.gdprPrivacy;
  return analytics.track('CMP Submitted', _extends({}, TCF_TRACK_PROPERTIES, getConfig('tcfTrackDefaultProperties')), {
    gdpr_privacy: gdprPrivacy.analytics,
    gdpr_privacy_advertising: gdprPrivacy.advertising
  });
};

/**
 * Get if we have user consents
 * @return {Promise<Object>}
 */
export var getGdprPrivacyValue = function getGdprPrivacyValue() {
  // try to get the actual gdprPrivacyValue and just return it
  var gdprPrivacyValue = gdprState.get();
  if (gdprPrivacyValue !== undefined) return Promise.resolve(gdprPrivacyValue);

  // // if we don't have a gdprPrivacyValue, then subscribe to it until we have a value
  return new Promise(function (resolve) {
    gdprState.addListener(function (gdprPrivacyValue) {
      return resolve(gdprPrivacyValue);
    });
  });
};

/**
 * Check if gdprPrivacyValue is accepted
 * @return {boolean}
 */
export var checkAnalyticsGdprIsAccepted = function checkAnalyticsGdprIsAccepted(gdprPrivacyValue) {
  return gdprPrivacyValue.analytics === USER_GDPR.ACCEPTED;
};

/**
 * Set gdprState according to list of purpose consents
 * @returns {string}
 */
var setGdprStateBy = function setGdprStateBy(purposeConsents) {
  var hasAnalyticsConsents = checkHasUserConsentedAnalytics(purposeConsents);
  var hasAdvertisingConsents = checkHasUserConsentedAdvertising(purposeConsents);
  // get the state key according to the gdprPrivacyValue
  var gdprAnalyticsStateKey = hasAnalyticsConsents ? USER_GDPR.ACCEPTED : USER_GDPR.DECLINED;
  var gdprAdvertisingStateKey = hasAdvertisingConsents ? USER_GDPR.ACCEPTED : USER_GDPR.DECLINED;
  // update gdprState with the new value for user gdpr
  gdprState.set({
    analytics: gdprAnalyticsStateKey,
    advertising: gdprAdvertisingStateKey
  });
  // return the new gdprState
  return {
    analytics: gdprAnalyticsStateKey,
    advertising: gdprAdvertisingStateKey
  };
};

/**
 * Read and decode the tcf cookie
 */
var getConsentsFromCookie = function getConsentsFromCookie() {
  var cookieValue = readCookie(TCF_COOKIE_KEY);
  if (!cookieValue) return;
  try {
    var _JSON$parse = JSON.parse(window.atob(cookieValue)),
      purpose = _JSON$parse.purpose;
    var consents = purpose.consents;
    return consents;
  } catch (e) {
    console.error(e);
  }
};

/**
 * Set the correct initial gdprState based on the consents cookie string
 */
var initializeGdprState = function initializeGdprState() {
  var consents = getConsentsFromCookie();
  if (consents) setGdprStateBy(consents);
};

/**
 * Sets global isFirstVisit flag based on the consents cookie string
 */
var initializeIsFirstVisit = function initializeIsFirstVisit() {
  var consents = getConsentsFromCookie();
  setConfig('isFirstVisit', !consents);
};

/**
 * Init TCF Tracking User Consents with Segment
 */
export default function initTcfTracking() {
  // first check if we're on server, as this doesn't work on server-side
  if (!isClient) return;
  // read the cookie and put the correct usergdprValue and isFirstVisit flag before listening events
  initializeGdprState();
  initializeIsFirstVisit();
  // do some checks before initializing tcf tracking as we do that only if available
  if (!checkTcfIsAvailable()) {
    var _gdprState$get;
    // if we don't have a gdpr state and tcf is not available
    // we should assume we don't known if we have consents
    var analyticsGdprState = (_gdprState$get = gdprState.get()) == null ? void 0 : _gdprState$get.analytics;
    if (analyticsGdprState === undefined) gdprState.set({
      analytics: USER_GDPR.UNKNOWN,
      advertising: USER_GDPR.UNKNOWN
    });
    // and we stop executing as we can't track tcf
    return;
  }
  // add a flag to the segmentWrapper config to know it's already initialized
  setConfig('initialized', true);

  // listen events from tcf api
  window.__tcfapi('addEventListener', TCF_API_VERSION, function (_ref3, success) {
    var eventStatus = _ref3.eventStatus,
      purpose = _ref3.purpose;
    if (!success) return Promise.resolve();

    // if we've already user consents or the user is accepting or declining now
    // we change the state of the GDPR to use in our tracking
    if (eventStatus === TCF_EVENTS.USER_ACTION_COMPLETE || eventStatus === TCF_EVENTS.LOADED) {
      var consents = purpose.consents;
      var gdprStateKey = setGdprStateBy(consents);
      // if it's a user action, then we will track it
      if (eventStatus === TCF_EVENTS.USER_ACTION_COMPLETE) {
        // extract the eventId and gdprPrivacy string to send with the track
        var analyticsStateKey = gdprStateKey.analytics,
          advertisingStateKey = gdprStateKey.advertising;
        var gdprPrivacy = {
          analytics: analyticsStateKey,
          advertising: advertisingStateKey
        };
        return trackTcf({
          gdprPrivacy: gdprPrivacy
        });
      }
    }
  });
}